@@ -162,16 +162,16 @@ static inline void cleanup_user_class_data(zend_class_entry *ce TSRMLS_DC)
 		zend_hash_apply(&ce->function_table, (apply_func_t) zend_cleanup_function_data_full TSRMLS_CC);
 	}
 	if (ce->static_members_table) {
-		zval **static_members = ce->static_members_table;
-		int count = ce->default_static_members_count;
 		int i;
 
-		ce->default_static_members_count = 0;
-		ce->default_static_members_table = ce->static_members_table = NULL;
-		for (i = 0; i < count; i++) {
-			zval_ptr_dtor(&static_members[i]);
+		for (i = 0; i < ce->default_static_members_count; i++) {
+			if (ce->static_members_table[i]) {
+				zval *p = ce->static_members_table[i];
+				ce->static_members_table[i] = NULL;
+				zval_ptr_dtor(&p);
+			}
 		}
-		efree(static_members);
+		ce->static_members_table = NULL;
 	}
 }
 
@@ -256,14 +256,9 @@ void _destroy_zend_class_traits_info(zend_class_entry *ce)
 			efree(ce->trait_precedences[i]->trait_method);
 
 			if (ce->trait_precedences[i]->exclude_from_classes) {
-				zend_uint j = 0;
-				zend_trait_precedence *cur_precedence = ce->trait_precedences[i];
-				while (cur_precedence->exclude_from_classes[j]) {
-					efree(cur_precedence->exclude_from_classes[j]);
-					j++;
-				}
 				efree(ce->trait_precedences[i]->exclude_from_classes);
 			}
+
 			efree(ce->trait_precedences[i]);
 			i++;
 		}
@@ -606,11 +601,11 @@ static void zend_resolve_finally_call(zend_op_array *op_array, zend_uint op_num,
 			while (i > 0) {
 				i--;
 				if (op_array->try_catch_array[i].finally_op &&
-				    op_num >= op_array->try_catch_array[i].try_op &&
-				    op_num < op_array->try_catch_array[i].finally_op - 1 &&
-				    (dst_num < op_array->try_catch_array[i].try_op ||
-				     dst_num > op_array->try_catch_array[i].finally_end)) {
-					
+					op_num >= op_array->try_catch_array[i].try_op &&
+					op_num < op_array->try_catch_array[i].finally_op - 1 &&
+					(dst_num < op_array->try_catch_array[i].try_op ||
+					 dst_num > op_array->try_catch_array[i].finally_end)) {
+
 					opline = get_next_op(op_array TSRMLS_CC);
 					opline->opcode = ZEND_FAST_CALL;
 					SET_UNUSED(opline->op1);
@@ -630,7 +625,7 @@ static void zend_resolve_finally_call(zend_op_array *op_array, zend_uint op_num,
 			SET_UNUSED(opline->op2);
 			opline->op1.opline_num = start_op;
 
-		    break;
+			break;
 		}
 	}	
 }
@@ -779,7 +774,7 @@ ZEND_API int pass_two(zend_op_array *op_array TSRMLS_DC)
 				if (op_array->fn_flags & ZEND_ACC_GENERATOR) {
 					if (opline->op1_type != IS_CONST || Z_TYPE_P(opline->op1.zv) != IS_NULL) {
 						CG(zend_lineno) = opline->lineno;
-						zend_error(E_COMPILE_ERROR, "Generators cannot return values using \"return\"");
+						zend_error_noreturn(E_COMPILE_ERROR, "Generators cannot return values using \"return\"");
 					}
 
 					opline->opcode = ZEND_GENERATOR_RETURN;
@@ -832,6 +827,9 @@ ZEND_API binary_op_type get_binary_op(int opcode)
 		case ZEND_ASSIGN_MUL:
 			return (binary_op_type) mul_function;
 			break;
+		case ZEND_POW:
+			return (binary_op_type) pow_function;
+			break;
 		case ZEND_DIV:
 		case ZEND_ASSIGN_DIV:
 			return (binary_op_type) div_function;
