@@ -1,27 +1,23 @@
 /*
- * The MIT License (MIT)
- *
- * Copyright (c) 2015 Derick Rethans
- *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to deal
- * in the Software without restriction, including without limitation the rights
- * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
- * THE SOFTWARE.
+   +----------------------------------------------------------------------+
+   | PHP Version 5                                                        |
+   +----------------------------------------------------------------------+
+   | Copyright (c) 1997-2015 The PHP Group                                |
+   +----------------------------------------------------------------------+
+   | This source file is subject to version 3.01 of the PHP license,      |
+   | that is bundled with this package in the file LICENSE, and is        |
+   | available through the world-wide-web at the following url:           |
+   | http://www.php.net/license/3_01.txt                                  |
+   | If you did not receive a copy of the PHP license and are unable to   |
+   | obtain it through the world-wide-web, please send a note to          |
+   | license@php.net so we can mail you a copy immediately.               |
+   +----------------------------------------------------------------------+
+   | Authors: Derick Rethans <derick@derickrethans.nl>                    |
+   +----------------------------------------------------------------------+
  */
 
+/* $Id$ */
+
 #include "timelib.h"
 
 #include <stdio.h>
@@ -35,14 +31,12 @@
 #else
 #include <strings.h>
 #endif
-
-#define TIMELIB_SUPPORTS_V2DATA
 #include "timezonedb.h"
 
 #if (defined(__APPLE__) || defined(__APPLE_CC__)) && (defined(__BIG_ENDIAN__) || defined(__LITTLE_ENDIAN__))
 # if defined(__LITTLE_ENDIAN__)
 #  undef WORDS_BIGENDIAN
-# else
+# else 
 #  if defined(__BIG_ENDIAN__)
 #   define WORDS_BIGENDIAN
 #  endif
@@ -55,14 +49,11 @@
 #define timelib_conv_int(l) ((l & 0x000000ff) << 24) + ((l & 0x0000ff00) << 8) + ((l & 0x00ff0000) >> 8) + ((l & 0xff000000) >> 24)
 #endif
 
-static int read_preamble(const unsigned char **tzf, timelib_tzinfo *tz)
+static void read_preamble(const unsigned char **tzf, timelib_tzinfo *tz)
 {
-	uint32_t version;
-
-	/* read ID */
-	version = (*tzf)[3] - '0';
+	/* skip ID */
 	*tzf += 4;
-
+	
 	/* read BC flag */
 	tz->bc = (**tzf == '\1');
 	*tzf += 1;
@@ -72,10 +63,8 @@ static int read_preamble(const unsigned char **tzf, timelib_tzinfo *tz)
 	tz->location.country_code[2] = '\0';
 	*tzf += 2;
 
-	/* skip rest of preamble */
+	/* skip read of preamble */
 	*tzf += 13;
-
-	return version;
 }
 
 static void read_header(const unsigned char **tzf, timelib_tzinfo *tz)
@@ -83,163 +72,128 @@ static void read_header(const unsigned char **tzf, timelib_tzinfo *tz)
 	uint32_t buffer[6];
 
 	memcpy(&buffer, *tzf, sizeof(buffer));
-	tz->bit32.ttisgmtcnt = timelib_conv_int(buffer[0]);
-	tz->bit32.ttisstdcnt = timelib_conv_int(buffer[1]);
-	tz->bit32.leapcnt    = timelib_conv_int(buffer[2]);
-	tz->bit32.timecnt    = timelib_conv_int(buffer[3]);
-	tz->bit32.typecnt    = timelib_conv_int(buffer[4]);
-	tz->bit32.charcnt    = timelib_conv_int(buffer[5]);
+	tz->ttisgmtcnt = timelib_conv_int(buffer[0]);
+	tz->ttisstdcnt = timelib_conv_int(buffer[1]);
+	tz->leapcnt    = timelib_conv_int(buffer[2]);
+	tz->timecnt    = timelib_conv_int(buffer[3]);
+	tz->typecnt    = timelib_conv_int(buffer[4]);
+	tz->charcnt    = timelib_conv_int(buffer[5]);
 	*tzf += sizeof(buffer);
 }
 
-static void skip_64bit_transistions(const unsigned char **tzf, timelib_tzinfo *tz)
-{
-	if (tz->bit64.timecnt) {
-		*tzf += (sizeof(int64_t) * tz->bit64.timecnt);
-		*tzf += (sizeof(unsigned char) * tz->bit64.timecnt);
-	}
-}
-
 static void read_transistions(const unsigned char **tzf, timelib_tzinfo *tz)
 {
 	int32_t *buffer = NULL;
 	uint32_t i;
 	unsigned char *cbuffer = NULL;
 
-	if (tz->bit32.timecnt) {
-		buffer = (int32_t*) timelib_malloc(tz->bit32.timecnt * sizeof(int32_t));
+	if (tz->timecnt) {
+		buffer = (int32_t*) malloc(tz->timecnt * sizeof(int32_t));
 		if (!buffer) {
 			return;
 		}
-		memcpy(buffer, *tzf, sizeof(int32_t) * tz->bit32.timecnt);
-		*tzf += (sizeof(int32_t) * tz->bit32.timecnt);
-		for (i = 0; i < tz->bit32.timecnt; i++) {
+		memcpy(buffer, *tzf, sizeof(int32_t) * tz->timecnt);
+		*tzf += (sizeof(int32_t) * tz->timecnt);
+		for (i = 0; i < tz->timecnt; i++) {
 			buffer[i] = timelib_conv_int(buffer[i]);
 		}
 
-		cbuffer = (unsigned char*) timelib_malloc(tz->bit32.timecnt * sizeof(unsigned char));
+		cbuffer = (unsigned char*) malloc(tz->timecnt * sizeof(unsigned char));
 		if (!cbuffer) {
-			timelib_free(buffer);
+			free(buffer);
 			return;
 		}
-		memcpy(cbuffer, *tzf, sizeof(unsigned char) * tz->bit32.timecnt);
-		*tzf += sizeof(unsigned char) * tz->bit32.timecnt;
+		memcpy(cbuffer, *tzf, sizeof(unsigned char) * tz->timecnt);
+		*tzf += sizeof(unsigned char) * tz->timecnt;
 	}
-
+	
 	tz->trans = buffer;
 	tz->trans_idx = cbuffer;
 }
 
-static void skip_64bit_types(const unsigned char **tzf, timelib_tzinfo *tz)
-{
-	*tzf += sizeof(unsigned char) * 6 * tz->bit64.typecnt;
-	*tzf += sizeof(char) * tz->bit64.charcnt;
-	if (tz->bit64.leapcnt) {
-		*tzf += sizeof(int64_t) * tz->bit64.leapcnt * 2;
-	}
-	if (tz->bit64.ttisstdcnt) {
-		*tzf += sizeof(unsigned char) * tz->bit64.ttisstdcnt;
-	}
-	if (tz->bit64.ttisgmtcnt) {
-		*tzf += sizeof(unsigned char) * tz->bit64.ttisgmtcnt;
-	}
-}
-
 static void read_types(const unsigned char **tzf, timelib_tzinfo *tz)
 {
 	unsigned char *buffer;
 	int32_t *leap_buffer;
 	unsigned int i, j;
 
-	buffer = (unsigned char*) timelib_malloc(tz->bit32.typecnt * sizeof(unsigned char) * 6);
+	buffer = (unsigned char*) malloc(tz->typecnt * sizeof(unsigned char) * 6);
 	if (!buffer) {
 		return;
 	}
-	memcpy(buffer, *tzf, sizeof(unsigned char) * 6 * tz->bit32.typecnt);
-	*tzf += sizeof(unsigned char) * 6 * tz->bit32.typecnt;
+	memcpy(buffer, *tzf, sizeof(unsigned char) * 6 * tz->typecnt);
+	*tzf += sizeof(unsigned char) * 6 * tz->typecnt;
 
-	tz->type = (ttinfo*) timelib_malloc(tz->bit32.typecnt * sizeof(struct ttinfo));
+	tz->type = (ttinfo*) malloc(tz->typecnt * sizeof(struct ttinfo));
 	if (!tz->type) {
-		timelib_free(buffer);
+		free(buffer);
 		return;
 	}
 
-	for (i = 0; i < tz->bit32.typecnt; i++) {
+	for (i = 0; i < tz->typecnt; i++) {
 		j = i * 6;
 		tz->type[i].offset = (buffer[j] * 16777216) + (buffer[j + 1] * 65536) + (buffer[j + 2] * 256) + buffer[j + 3];
 		tz->type[i].isdst = buffer[j + 4];
 		tz->type[i].abbr_idx = buffer[j + 5];
 	}
-	timelib_free(buffer);
+	free(buffer);
 
-	tz->timezone_abbr = (char*) timelib_malloc(tz->bit32.charcnt);
+	tz->timezone_abbr = (char*) malloc(tz->charcnt);
 	if (!tz->timezone_abbr) {
 		return;
 	}
-	memcpy(tz->timezone_abbr, *tzf, sizeof(char) * tz->bit32.charcnt);
-	*tzf += sizeof(char) * tz->bit32.charcnt;
+	memcpy(tz->timezone_abbr, *tzf, sizeof(char) * tz->charcnt);
+	*tzf += sizeof(char) * tz->charcnt;
 
-	if (tz->bit32.leapcnt) {
-		leap_buffer = (int32_t *) timelib_malloc(tz->bit32.leapcnt * 2 * sizeof(int32_t));
+	if (tz->leapcnt) {
+		leap_buffer = (int32_t *) malloc(tz->leapcnt * 2 * sizeof(int32_t));
 		if (!leap_buffer) {
 			return;
 		}
-		memcpy(leap_buffer, *tzf, sizeof(int32_t) * tz->bit32.leapcnt * 2);
-		*tzf += sizeof(int32_t) * tz->bit32.leapcnt * 2;
+		memcpy(leap_buffer, *tzf, sizeof(int32_t) * tz->leapcnt * 2);
+		*tzf += sizeof(int32_t) * tz->leapcnt * 2;
 
-		tz->leap_times = (tlinfo*) timelib_malloc(tz->bit32.leapcnt * sizeof(tlinfo));
+		tz->leap_times = (tlinfo*) malloc(tz->leapcnt * sizeof(tlinfo));
 		if (!tz->leap_times) {
-			timelib_free(leap_buffer);
+			free(leap_buffer);
 			return;
 		}
-		for (i = 0; i < tz->bit32.leapcnt; i++) {
+		for (i = 0; i < tz->leapcnt; i++) {
 			tz->leap_times[i].trans = timelib_conv_int(leap_buffer[i * 2]);
 			tz->leap_times[i].offset = timelib_conv_int(leap_buffer[i * 2 + 1]);
 		}
-		timelib_free(leap_buffer);
+		free(leap_buffer);
 	}
 
-	if (tz->bit32.ttisstdcnt) {
-		buffer = (unsigned char*) timelib_malloc(tz->bit32.ttisstdcnt * sizeof(unsigned char));
+	if (tz->ttisstdcnt) {
+		buffer = (unsigned char*) malloc(tz->ttisstdcnt * sizeof(unsigned char));
 		if (!buffer) {
 			return;
 		}
-		memcpy(buffer, *tzf, sizeof(unsigned char) * tz->bit32.ttisstdcnt);
-		*tzf += sizeof(unsigned char) * tz->bit32.ttisstdcnt;
+		memcpy(buffer, *tzf, sizeof(unsigned char) * tz->ttisstdcnt);
+		*tzf += sizeof(unsigned char) * tz->ttisstdcnt;
 
-		for (i = 0; i < tz->bit32.ttisstdcnt; i++) {
+		for (i = 0; i < tz->ttisstdcnt; i++) {
 			tz->type[i].isstdcnt = buffer[i];
 		}
-		timelib_free(buffer);
+		free(buffer);
 	}
 
-	if (tz->bit32.ttisgmtcnt) {
-		buffer = (unsigned char*) timelib_malloc(tz->bit32.ttisgmtcnt * sizeof(unsigned char));
+	if (tz->ttisgmtcnt) {
+		buffer = (unsigned char*) malloc(tz->ttisgmtcnt * sizeof(unsigned char));
 		if (!buffer) {
 			return;
 		}
-		memcpy(buffer, *tzf, sizeof(unsigned char) * tz->bit32.ttisgmtcnt);
-		*tzf += sizeof(unsigned char) * tz->bit32.ttisgmtcnt;
+		memcpy(buffer, *tzf, sizeof(unsigned char) * tz->ttisgmtcnt);
+		*tzf += sizeof(unsigned char) * tz->ttisgmtcnt;
 
-		for (i = 0; i < tz->bit32.ttisgmtcnt; i++) {
+		for (i = 0; i < tz->ttisgmtcnt; i++) {
 			tz->type[i].isgmtcnt = buffer[i];
 		}
-		timelib_free(buffer);
+		free(buffer);
 	}
 }
 
-static void skip_posix_string(const unsigned char **tzf, timelib_tzinfo *tz)
-{
-	int n_count = 0;
-
-	do {
-		if (*tzf[0] == '\n') {
-			n_count++;
-		}
-		(*tzf)++;
-	} while (n_count < 2);
-}
-
 static void read_location(const unsigned char **tzf, timelib_tzinfo *tz)
 {
 	uint32_t buffer[3];
@@ -253,7 +207,7 @@ static void read_location(const unsigned char **tzf, timelib_tzinfo *tz)
 	comments_len = timelib_conv_int(buffer[2]);
 	*tzf += sizeof(buffer);
 
-	tz->location.comments = timelib_malloc(comments_len + 1);
+	tz->location.comments = malloc(comments_len + 1);
 	memcpy(tz->location.comments, *tzf, comments_len);
 	tz->location.comments[comments_len] = '\0';
 	*tzf += comments_len;
@@ -267,12 +221,12 @@ void timelib_dump_tzinfo(timelib_tzinfo *tz)
 	printf("Geo Location:      %f,%f\n", tz->location.latitude, tz->location.longitude);
 	printf("Comments:\n%s\n",          tz->location.comments);
 	printf("BC:                %s\n",  tz->bc ? "" : "yes");
-	printf("UTC/Local count:   " TIMELIB_ULONG_FMT "\n", (timelib_ulong) tz->bit32.ttisgmtcnt);
-	printf("Std/Wall count:    " TIMELIB_ULONG_FMT "\n", (timelib_ulong) tz->bit32.ttisstdcnt);
-	printf("Leap.sec. count:   " TIMELIB_ULONG_FMT "\n", (timelib_ulong) tz->bit32.leapcnt);
-	printf("Trans. count:      " TIMELIB_ULONG_FMT "\n", (timelib_ulong) tz->bit32.timecnt);
-	printf("Local types count: " TIMELIB_ULONG_FMT "\n", (timelib_ulong) tz->bit32.typecnt);
-	printf("Zone Abbr. count:  " TIMELIB_ULONG_FMT "\n", (timelib_ulong) tz->bit32.charcnt);
+	printf("UTC/Local count:   %lu\n", (unsigned long) tz->ttisgmtcnt);
+	printf("Std/Wall count:    %lu\n", (unsigned long) tz->ttisstdcnt);
+	printf("Leap.sec. count:   %lu\n", (unsigned long) tz->leapcnt);
+	printf("Trans. count:      %lu\n", (unsigned long) tz->timecnt);
+	printf("Local types count: %lu\n", (unsigned long) tz->typecnt);
+	printf("Zone Abbr. count:  %lu\n", (unsigned long) tz->charcnt);
 
 	printf ("%8s (%12s) = %3d [%5ld %1d %3d '%s' (%d,%d)]\n",
 		"", "", 0,
@@ -283,7 +237,7 @@ void timelib_dump_tzinfo(timelib_tzinfo *tz)
 		tz->type[0].isstdcnt,
 		tz->type[0].isgmtcnt
 		);
-	for (i = 0; i < tz->bit32.timecnt; i++) {
+	for (i = 0; i < tz->timecnt; i++) {
 		printf ("%08X (%12d) = %3d [%5ld %1d %3d '%s' (%d,%d)]\n",
 			tz->trans[i], tz->trans[i], tz->trans_idx[i],
 			(long int) tz->type[tz->trans_idx[i]].offset,
@@ -294,7 +248,7 @@ void timelib_dump_tzinfo(timelib_tzinfo *tz)
 			tz->type[tz->trans_idx[i]].isgmtcnt
 			);
 	}
-	for (i = 0; i < tz->bit32.leapcnt; i++) {
+	for (i = 0; i < tz->leapcnt; i++) {
 		printf ("%08X (%12ld) = %d\n",
 			tz->leap_times[i].trans,
 			(long) tz->leap_times[i].trans,
@@ -310,10 +264,10 @@ static int seek_to_tz_position(const unsigned char **tzf, char *timezone, const
 
 	tmp = setlocale(LC_CTYPE, NULL);
 	if (tmp) {
-		cur_locale = timelib_strdup(tmp);
+		cur_locale = strdup(tmp);
 	}
 	setlocale(LC_CTYPE, "C");
-#endif
+#endif	
 
 	do {
 		int mid = ((unsigned)left + right) >> 1;
@@ -327,8 +281,8 @@ static int seek_to_tz_position(const unsigned char **tzf, char *timezone, const
 			(*tzf) = &(tzdb->data[tzdb->index[mid].pos]);
 #ifdef HAVE_SETLOCALE
 			setlocale(LC_CTYPE, cur_locale);
-			if (cur_locale) timelib_free(cur_locale);
-#endif
+			if (cur_locale) free(cur_locale);
+#endif	
 			return 1;
 		}
 
@@ -336,8 +290,8 @@ static int seek_to_tz_position(const unsigned char **tzf, char *timezone, const
 
 #ifdef HAVE_SETLOCALE
 	setlocale(LC_CTYPE, cur_locale);
-	if (cur_locale) timelib_free(cur_locale);
-#endif
+	if (cur_locale) free(cur_locale);
+#endif	
 	return 0;
 }
 
@@ -358,45 +312,18 @@ int timelib_timezone_id_is_valid(char *timezone, const timelib_tzdb *tzdb)
 	return (seek_to_tz_position(&tzf, timezone, tzdb));
 }
 
-static void skip_64bit_preamble(const unsigned char **tzf, timelib_tzinfo *tz)
-{
-	*tzf += 20;
-}
-
-static void read_64bit_header(const unsigned char **tzf, timelib_tzinfo *tz)
-{
-	uint32_t buffer[6];
-
-	memcpy(&buffer, *tzf, sizeof(buffer));
-	tz->bit64.ttisgmtcnt = timelib_conv_int(buffer[0]);
-	tz->bit64.ttisstdcnt = timelib_conv_int(buffer[1]);
-	tz->bit64.leapcnt    = timelib_conv_int(buffer[2]);
-	tz->bit64.timecnt    = timelib_conv_int(buffer[3]);
-	tz->bit64.typecnt    = timelib_conv_int(buffer[4]);
-	tz->bit64.charcnt    = timelib_conv_int(buffer[5]);
-	*tzf += sizeof(buffer);
-}
-
 timelib_tzinfo *timelib_parse_tzfile(char *timezone, const timelib_tzdb *tzdb)
 {
 	const unsigned char *tzf;
 	timelib_tzinfo *tmp;
-	int version;
 
 	if (seek_to_tz_position(&tzf, timezone, tzdb)) {
 		tmp = timelib_tzinfo_ctor(timezone);
 
-		version = read_preamble(&tzf, tmp);
+		read_preamble(&tzf, tmp);
 		read_header(&tzf, tmp);
 		read_transistions(&tzf, tmp);
 		read_types(&tzf, tmp);
-		if (version == 2) {
-			skip_64bit_preamble(&tzf, tmp);
-			read_64bit_header(&tzf, tmp);
-			skip_64bit_transistions(&tzf, tmp);
-			skip_64bit_types(&tzf, tmp);
-			skip_posix_string(&tzf, tmp);
-		}
 		read_location(&tzf, tmp);
 	} else {
 		tmp = NULL;
@@ -411,9 +338,9 @@ static ttinfo* fetch_timezone_offset(timelib_tzinfo *tz, timelib_sll ts, timelib
 
 	/* If there is no transition time, we pick the first one, if that doesn't
 	 * exist we return NULL */
-	if (!tz->bit32.timecnt || !tz->trans) {
+	if (!tz->timecnt || !tz->trans) {
 		*transition_time = 0;
-		if (tz->bit32.typecnt == 1) {
+		if (tz->typecnt == 1) {
 			return &(tz->type[0]);
 		}
 		return NULL;
@@ -428,36 +355,36 @@ static ttinfo* fetch_timezone_offset(timelib_tzinfo *tz, timelib_sll ts, timelib
 
 		*transition_time = 0;
 		j = 0;
-		while (j < tz->bit32.timecnt && tz->type[tz->trans_idx[j]].isdst) {
+		while (j < tz->timecnt && tz->type[j].isdst) {
 			++j;
 		}
-		if (j == tz->bit32.timecnt) {
+		if (j == tz->timecnt) {
 			j = 0;
 		}
-		return &(tz->type[tz->trans_idx[j]]);
+		return &(tz->type[j]);
 	}
 
 	/* In all other cases we loop through the available transtion times to find
 	 * the correct entry */
-	for (i = 0; i < tz->bit32.timecnt; i++) {
+	for (i = 0; i < tz->timecnt; i++) {
 		if (ts < tz->trans[i]) {
 			*transition_time = tz->trans[i - 1];
 			return &(tz->type[tz->trans_idx[i - 1]]);
 		}
 	}
-	*transition_time = tz->trans[tz->bit32.timecnt - 1];
-	return &(tz->type[tz->trans_idx[tz->bit32.timecnt - 1]]);
+	*transition_time = tz->trans[tz->timecnt - 1];
+	return &(tz->type[tz->trans_idx[tz->timecnt - 1]]);
 }
 
 static tlinfo* fetch_leaptime_offset(timelib_tzinfo *tz, timelib_sll ts)
 {
 	int i;
 
-	if (!tz->bit32.leapcnt || !tz->leap_times) {
+	if (!tz->leapcnt || !tz->leap_times) {
 		return NULL;
 	}
 
-	for (i = tz->bit32.leapcnt - 1; i > 0; i--) {
+	for (i = tz->leapcnt - 1; i > 0; i--) {
 		if (ts > tz->leap_times[i].trans) {
 			return &(tz->leap_times[i]);
 		}
@@ -469,7 +396,7 @@ int timelib_timestamp_is_in_dst(timelib_sll ts, timelib_tzinfo *tz)
 {
 	ttinfo *to;
 	timelib_sll dummy;
-
+	
 	if ((to = fetch_timezone_offset(tz, ts, &dummy))) {
 		return to->isdst;
 	}
@@ -503,7 +430,7 @@ timelib_time_offset *timelib_get_time_zone_info(timelib_sll ts, timelib_tzinfo *
 
 	tmp->offset = offset;
 	tmp->leap_secs = leap_secs;
-	tmp->abbr = abbr ? timelib_strdup(abbr) : timelib_strdup("GMT");
+	tmp->abbr = abbr ? strdup(abbr) : strdup("GMT");
 
 	return tmp;
 }
@@ -512,12 +439,12 @@ timelib_sll timelib_get_current_offset(timelib_time *t)
 {
 	timelib_time_offset *gmt_offset;
 	timelib_sll retval;
-
+			
 	switch (t->zone_type) {
 		case TIMELIB_ZONETYPE_ABBR:
 		case TIMELIB_ZONETYPE_OFFSET:
 			return (t->z + t->dst) * -60;
-
+			
 		case TIMELIB_ZONETYPE_ID:
 			gmt_offset = timelib_get_time_zone_info(t->sse, t->tz_info);
 			retval = gmt_offset->offset;
