@@ -35,7 +35,7 @@
 #include "zend_stack.h"
 #include "php_output.h"
 
-ZEND_DECLARE_MODULE_GLOBALS(output);
+PHPAPI ZEND_DECLARE_MODULE_GLOBALS(output);
 
 const char php_output_default_handler_name[sizeof("default output handler")] = "default output handler";
 const char php_output_devnull_handler_name[sizeof("null output handler")] = "null output handler";
@@ -172,22 +172,21 @@ PHPAPI void php_output_deactivate(TSRMLS_D)
 {
 	php_output_handler **handler = NULL;
 
-	if ((OG(flags) & PHP_OUTPUT_ACTIVATED)) {
-		php_output_header(TSRMLS_C);
+	php_output_header(TSRMLS_C);
 
-		OG(flags) ^= PHP_OUTPUT_ACTIVATED;
-		OG(active) = NULL;
-		OG(running) = NULL;
+	OG(flags) ^= PHP_OUTPUT_ACTIVATED;
+	OG(active) = NULL;
+	OG(running) = NULL;
 
-		/* release all output handlers */
-		if (OG(handlers).elements) {
-			while (SUCCESS == zend_stack_top(&OG(handlers), (void *) &handler)) {
-				php_output_handler_free(handler TSRMLS_CC);
-				zend_stack_del_top(&OG(handlers));
-			}
-			zend_stack_destroy(&OG(handlers));
+	/* release all output handlers */
+	if (OG(handlers).elements) {
+		while (SUCCESS == zend_stack_top(&OG(handlers), (void *) &handler)) {
+			php_output_handler_free(handler TSRMLS_CC);
+			zend_stack_del_top(&OG(handlers));
 		}
+		zend_stack_destroy(&OG(handlers));
 	}
+
 }
 /* }}} */
 
