@@ -1390,12 +1390,10 @@ char * LSAPI_GetHeader_r( LSAPI_Request * pReq, int headerIndex )
     off = pReq->m_pHeaderIndex->m_headerOff[ headerIndex ];
     if ( !off )
         return NULL;
-    if ( *(pReq->m_pHttpHeader + off 
-        + pReq->m_pHeaderIndex->m_headerLen[ headerIndex ]) )
-    {
-        *( pReq->m_pHttpHeader + off 
-            + pReq->m_pHeaderIndex->m_headerLen[ headerIndex ]) = 0;
-    }
+        if ( *(pReq->m_pHttpHeader + off + 
+                 pReq->m_pHeaderIndex->m_headerLen[ headerIndex ]) )
+                *( pReq->m_pHttpHeader + off + 
+                        pReq->m_pHeaderIndex->m_headerLen[ headerIndex ]) = 0;
     return pReq->m_pHttpHeader + off;
 }
 
@@ -1832,21 +1830,12 @@ ssize_t LSAPI_Write_Stderr_r( LSAPI_Request * pReq, const char * pBuf, size_t le
 static char * GetHeaderVar( LSAPI_Request * pReq, const char * name )
 {
     int i;
-    char * pValue;
     for( i = 0; i < H_TRANSFER_ENCODING; ++i )
     {
         if ( pReq->m_pHeaderIndex->m_headerOff[i] )
         {
             if ( strcmp( name, CGI_HEADERS[i] ) == 0 )
-            {
-                pValue = pReq->m_pHttpHeader 
-                         + pReq->m_pHeaderIndex->m_headerOff[i];
-                if ( *(pValue + pReq->m_pHeaderIndex->m_headerLen[i]) != '\0')
-                {
-                    *(pValue + pReq->m_pHeaderIndex->m_headerLen[i]) = '\0';
-                }
-                return pValue;
-            }
+                return pReq->m_pHttpHeader + pReq->m_pHeaderIndex->m_headerOff[i];
         }
     }
     if ( pReq->m_pHeader->m_cntUnknownHeaders > 0 )
@@ -1873,15 +1862,7 @@ static char * GetHeaderVar( LSAPI_Request * pReq, const char * name )
                 ++p; ++pKey;
             }
             if (( pKey == pKeyEnd )&& (!*p ))
-            {
-                pValue = pReq->m_pHttpHeader + pCur->valueOff;
-                
-                if ( *(pValue + pCur->valueLen) != '\0')
-                {
-                    *(pValue + pCur->valueLen) = '\0';
-                }
-                return pValue;
-            }
+                return pReq->m_pHttpHeader + pCur->valueOff;
             ++pCur;
         }
     }
@@ -2681,7 +2662,7 @@ static void lsapi_check_child_status( long tmCur )
     }
     if ( abs( g_prefork_server->m_iCurChildren - count ) > 1 )
     {
-        fprintf( stderr, "Children tracking is wrong: PID: %d, Cur Childen: %d, count: %d, idle: %d, dying: %d\n", getpid(), 
+        fprintf( stderr, "Children tracking is wrong: PID: %d, Cur Children: %d, count: %d, idle: %d, dying: %d\n", getpid(), 
                 g_prefork_server->m_iCurChildren, count, idle, dying );
 
     }
@@ -3131,20 +3112,10 @@ static int lsapi_initSuEXEC()
     if ( !s_defaultUid || !s_defaultGid )
     {
         pw = getpwnam( "nobody" );
-        if ( pw )
-        {
-            if ( !s_defaultUid )
-                s_defaultUid = pw->pw_uid;
-            if ( !s_defaultGid )
-                s_defaultGid = pw->pw_gid;
-        }
-        else
-        {
-            if ( !s_defaultUid )
-                s_defaultUid = 10000;
-            if ( !s_defaultGid )
-                s_defaultGid = 10000;
-        }
+        if ( !s_defaultUid )
+            s_defaultUid = pw->pw_uid;
+        if ( !s_defaultGid )
+            s_defaultGid = pw->pw_gid;
     }
     return 0;
 }
